@@ -239,7 +239,7 @@ getContractBlockValues = function(contract) {
       "Sum insured"         = contract$Parameters$ContractData$sumInsured,
       "Mortality table"     = contract$Parameters$ActuarialBases$mortalityTable@name,
       i                     = contract$Parameters$ActuarialBases$i,
-      "YOB"                 = contract$Parameters$ContractData$YOB,
+      "Birth Date"          = contract$Parameters$ContractData$birthDate,
       "Age"                 = contract$Parameters$ContractData$age,
       "Technical Age"       = contract$Parameters$ContractData$technicalAge,
       "Policy duration"     = contract$Parameters$ContractData$policyPeriod,
